#include <QString>
#include "gds_command.h"

//    GDSCommand *c = GDSCommandCreator::create(QString("AVAIL LON SIN"));


GDSCommand::GDSCommand(const QByteArray& command) {
    m_params = QString(command).trimmed().split(' ');
}

QString GDSCommand::getParam( int i )
{
	return m_params.value(i);
}

GDSCommandSellFlight::GDSCommandSellFlight(const QByteArray& command): GDSCommand(command)
{
	m_long = m_params.length() > 4;
}

GDSCommand* GDSCommandCreator::create(const QByteArray& command) {
    if ( command.startsWith( GDSCommandAvail::signature().toLocal8Bit() ) )
	{
        return new GDSCommandAvail(command);
    }
	else if( command.startsWith( GDSCommandJump::signature().toLocal8Bit() ) )
	{
        return new GDSCommandJump(command);
    }
	else if( command.startsWith( GDSCommandSellFlight::signature().toLocal8Bit() ) )
	{
        return new GDSCommandSellFlight(command);
    }
	else if( command.startsWith( GDSCommandSellHotel::signature().toLocal8Bit() ) )
	{
		return new GDSCommandSellHotel(command);
	}
	else if( command.startsWith( GDSCommandEndSell::signature().toLocal8Bit() ) )
	{
		return new GDSCommandEndSell(command);
	}
	else if( command.startsWith( GDSCommandHelp::signature().toLocal8Bit() ) )
	{
		return new GDSCommandHelp(command);
	}
	else if( command.startsWith( GDSCommandAddPassenger::signature().toLocal8Bit() ) )
	{
		return new GDSCommandAddPassenger(command);
	}
	else if( command.startsWith( GDSCommandAddFlight::signature().toLocal8Bit() ) )
	{
		return new GDSCommandAddFlight(command);
	}
	else if( command.startsWith( GDSCommandAddHotel::signature().toLocal8Bit() ) )
	{
		return new GDSCommandAddHotel(command);
	}
	else if( command.startsWith( GDSCommandAddUser::signature().toLocal8Bit() ) )
	{
		return new GDSCommandAddUser(command);
	}
	else if( command.startsWith( GDSCommandBatch::signature().toLocal8Bit() ) )
	{
		return new GDSCommandBatch(command);
	}
	else if( command.startsWith( GDSCommandRetrieve::signature().toLocal8Bit() ) )
	{
		return new GDSCommandRetrieve(command);
	}
	else if( command.startsWith( GDSCommandChangePassword::signature().toLocal8Bit() ) )
	{
		return new GDSCommandChangePassword(command);
	}
	else if( command.startsWith( GDSCommandHotelsList::signature().toLocal8Bit() ) )
	{
		return new GDSCommandHotelsList(command);
	}
	else if( command.startsWith( GDSCommandDeletePNR::signature().toLocal8Bit() ) )
	{
		return new GDSCommandDeletePNR(command);
	}
	else if( command.startsWith( GDSCommandAbortSell::signature().toLocal8Bit() ) )
	{
		return new GDSCommandAbortSell(command);
	}
	return new GDSCommand(command);
}

QString GDSCommandCreator::help( const QString& command )
{
	if ( command.startsWith( GDSCommandAvail::signature().trimmed() ) )
	{
		return GDSCommandAvail::help();
	}
	else if( command.startsWith( GDSCommandJump::signature().trimmed() ) )
	{
		return GDSCommandJump::help();
	}
	else if( command.startsWith( GDSCommandSellFlight::signature().trimmed() ) )
	{
		return GDSCommandSellFlight::help();
	}
	else if( command.startsWith( GDSCommandSellHotel::signature().trimmed() ) )
	{
		return GDSCommandSellHotel::help();
	}
	else if( command.startsWith( GDSCommandEndSell::signature().trimmed() ) )
	{
		return GDSCommandEndSell::help();
	}
	else if( command.startsWith( GDSCommandAbortSell::signature().trimmed() ) )
	{
		return GDSCommandAbortSell::help();
	}
	else if( command.startsWith( GDSCommandHelp::signature().trimmed() ) )
	{
		return GDSCommandHelp::help();
	}
	else if( command.startsWith( GDSCommandAddPassenger::signature().trimmed() ) )
	{
		return GDSCommandAddPassenger::help();
	}
	else if( command.startsWith( GDSCommandRetrieve::signature().trimmed() ) )
	{
		return GDSCommandRetrieve::help();
	}
	else if( command.startsWith( GDSCommandChangePassword::signature().trimmed() ) )
	{
		return GDSCommandChangePassword::help();
	}
	else if( command.startsWith( GDSCommandHotelsList::signature().trimmed() ) )
	{
		return GDSCommandHotelsList::help();
	}
	else if( command.startsWith( GDSCommandDeletePNR::signature().trimmed() ) )
	{
		return GDSCommandDeletePNR::help();
	}
	else if ( command.isEmpty() )
	{
		QStringList list;
		list << GDSCommandAvail::signature().trimmed() + " - " + GDSCommandAvail::help().split("\n").first();
		list << GDSCommandJump::signature().trimmed() + " - " + GDSCommandJump::help().split("\n").first();
		list << GDSCommandSellFlight::signature().trimmed() + " - " + GDSCommandSellFlight::help().split("\n").first();
		list << GDSCommandEndSell::signature().trimmed() + " - " + GDSCommandEndSell::help().split("\n").first();
		list << GDSCommandHelp::signature().trimmed() + " - " + GDSCommandHelp::help().split("\n").first();
		list << GDSCommandAddPassenger::signature().trimmed() + " - " + GDSCommandAddPassenger::help().split("\n").first();
		list << GDSCommandRetrieve::signature().trimmed() + " - " + GDSCommandRetrieve::help().split("\n").first();
		list << GDSCommandChangePassword::signature().trimmed() + " - " + GDSCommandChangePassword::help().split("\n").first();
		list << GDSCommandHotelsList::signature().trimmed() + " - " + GDSCommandHotelsList::help().split("\n").first();
		list << GDSCommandDeletePNR::signature().trimmed() + " - " + GDSCommandDeletePNR::help().split("\n").first();
		list << GDSCommandSellHotel::signature().trimmed() + " - " + GDSCommandSellHotel::help().split("\n").first();
		list << GDSCommandAbortSell::signature().trimmed() + " - " + GDSCommandAbortSell::help().split("\n").first();
		return "Help topics\n" + list.join("\n");
	}
	return GDSCommand::help();
}
