#ifndef GDSCOMMAND_H
#define GDSCOMMAND_H

#include <QStringList>
#include "datehandler.h"
#include "exception.h"

class GDSCommand {
public:
	GDSCommand(const QByteArray&);
	virtual ~GDSCommand() {};
	static QString help() { return "No help on subject"; }
	QString getParam( int i );
protected:
	QStringList m_params;
};

class GDSCommandAvail: public GDSCommand {
public:
	GDSCommandAvail(const QByteArray& command): GDSCommand(command) {}
	static QString signature() { return "A "; }
	static QString help() { return "Flights availability\nA <type> <date> <origin> <destination>"; }
	QString type() { return getParam( 1 ); }
	BAYDateTime date() { return BAYDateTime( getParam( 2 ) ); }
	QString origin() { return getParam( 3 ); }
	QString destination() { return getParam( 4 ); }
};

class GDSCommandJump: public GDSCommand {
public:
	GDSCommandJump(const QByteArray& command): GDSCommand(command) {}
	static QString help() { return "Jump In\nJI <login> <password>"; }
	static QString signature() { return "JI "; }
	QString login() { return getParam( 1 ); }
	QString password() { return getParam( 2 ); }
};

class GDSCommandSellFlight: public GDSCommand {
public:
	GDSCommandSellFlight(const QByteArray&);
	static QString signature() { return "SS "; }
	static QString help() { return "Sell Flight\nSS <carrier> <flight> <seat class> <date> <origin> <destination> <seat count>\nSS <seat count> <seat class> <availability line>"; }
	bool isLong() { return m_long; }
	int number() { bool ok = true; int result = m_long ? getParam( 7 ).toInt( &ok ) : getParam( 1 ).toInt( &ok ); check( ok ); return result; }
	QString sclass() { return m_long ? getParam( 3 ) : getParam( 2 ); }
	int line() { bool ok = true; int result = m_long ? -1 : getParam( 3 ).toInt( &ok ) - 1; check( ok ); return result; }
	QString carrier() { return m_long ? getParam( 1 ) : QString(); }
	QString flight() { return m_long ? getParam( 2 ) : QString(); }
	BAYDateTime date() { return m_long ? BAYDateTime( getParam( 4 ) ): BAYDateTime(); }
	QString origin() { return m_long ? getParam( 5 ) : QString(); }
	QString destination() { return m_long ? getParam( 6 ) : QString(); }
private:
	bool m_long;
};

class GDSCommandSellHotel: public GDSCommand {
public:
	GDSCommandSellHotel(const QByteArray& command): GDSCommand(command) {}
	static QString signature() { return "SH "; }
	static QString help() { return "Sell Hotel\nSH <room count> <room class> <availability line>"; }
	int number() { bool ok = true; int result = getParam( 1 ).toInt( &ok ); check( ok ); return result; }
	QString sclass() { return getParam( 2 ); }
	int line() { bool ok = true; int result = getParam( 3 ).toInt( &ok ) - 1; check( ok ); return result; }
};

class GDSCommandEndSell: public GDSCommand {
public:
	GDSCommandEndSell(const QByteArray& command): GDSCommand(command) {}
	static QString help() { return "End Sell Transaction\nET"; }
	static QString signature() { return "ET"; }
};


class GDSCommandAbortSell: public GDSCommand {
public:
	GDSCommandAbortSell(const QByteArray& command): GDSCommand(command) {}
	static QString help() { return "Abort Sell Transaction\nAT"; }
	static QString signature() { return "AT"; }
};

class GDSCommandHelp: public GDSCommand {
public:
	GDSCommandHelp(const QByteArray& command): GDSCommand(command) {}
	static QString help() { return "Help\nHE\nHE <command>"; }
	static QString signature() { return "HE"; }
	QString topic() { return getParam( 1 ); }
};

class GDSCommandAddPassenger: public GDSCommand {
public:
	GDSCommandAddPassenger(const QByteArray& command): GDSCommand(command) {}
	static QString signature() { return "NM "; }
	static QString help() { return "Add passenger/guest\nNM <first name> <last name> <title> <passport>"; }
	QString lastName() { return getParam( 1 ); }
	QString name() { return getParam( 2 ); }
	QString title() { return getParam( 3 ); }
	QString passport() { return getParam( 4 ); }
};

class GDSCommandAddHotel: public GDSCommand {
public:
	GDSCommandAddHotel(const QByteArray& command): GDSCommand(command) {}
	static QString signature() { return "AH "; }
	static QString help() { return "No help on subject"; }
	QString chain() { return getParam( 1 ); }
	QString name() { return getParam( 2 ); }
	QString city() { return getParam( 3 ); }
	BAYDateTime date() { return BAYDateTime( getParam( 4 ) ); }
	QString rooms() { return getParam( 5 ); }
};

class GDSCommandAddFlight: public GDSCommand {
public:
	GDSCommandAddFlight(const QByteArray& command): GDSCommand(command) {}
	static QString signature() { return "AF "; }
	static QString help() { return "No help on subject"; }
	QString airline() { return getParam( 1 ); }
	QString flightNum() { return getParam( 2 ); }
	QString orig() { return getParam( 3 ); }
	QString dest() { return getParam( 4 ); }
	BAYDateTime date() { return BAYDateTime( getParam( 5 ) + " " + getParam( 6 ) ); }
	QString seats() { return getParam( 7 ); }
};

class GDSCommandAddUser: public GDSCommand {
public:
	GDSCommandAddUser(const QByteArray& command): GDSCommand(command) {}
	static QString signature() { return "AU "; }
	static QString help() { return "No help on subject"; }
	QString login() { return getParam( 1 ); }
	QString password() { return getParam( 2 ); }
	bool changeable() { return getParam( 3 ) == "TRUE"; }
};

class GDSCommandBatch: public GDSCommand {
public:
	GDSCommandBatch(const QByteArray& command): GDSCommand(command) {}
	static QString signature() { return "BA "; }
	static QString help() { return "No help on subject"; }
	QString RSAsignature() { return getParam( 2 ); }
	int count() { bool ok = true; int result = getParam( 1 ).toInt( &ok ); check( ok ); return result; }
};

class GDSCommandRetrieve: public GDSCommand {
public:
	GDSCommandRetrieve(const QByteArray& command): GDSCommand(command) {}
	static QString signature() { return "RT "; }
	static QString help() { return "Retrieve PNR\nRT <PNR ID>"; }
	QString id() { return getParam( 1 ); }
};

class GDSCommandChangePassword: public GDSCommand {
public:
	GDSCommandChangePassword(const QByteArray& command): GDSCommand(command) {}
	static QString signature() { return "CP "; }
	static QString help() { return "Change password\nCP <new password>"; }
	QString password() { return getParam( 1 ); }
};

class GDSCommandHotelsList: public GDSCommand {
public:
	GDSCommandHotelsList(const QByteArray& command): GDSCommand(command) {}
	static QString signature() { return "HA "; }
	static QString help() { return "Hotels Availability\nHA <city> <date>"; }
	QString city() { return getParam( 1 ); }
	BAYDateTime date() { return BAYDateTime( getParam( 2 ) ); }
};

class GDSCommandDeletePNR: public GDSCommand {
public:
	GDSCommandDeletePNR(const QByteArray& command): GDSCommand(command) {}
	static QString signature() { return "DE "; }
	static QString help() { return "Delete PNR\nDE <PNR ID>"; }
	QString id() { return getParam( 1 ); }
};

class GDSCommandCreator {
public:
	static GDSCommand* create(const QByteArray&);
	static QString help(const QString&);
};

#endif // GDSCOMMAND_H