#include "gdsclientauthenticator.h"
#include "gdsclientconnection.h"
#include "logstream.h"
#include "gdsclient.h"
#include "gdsclient_airline.h"
#include "gdsclient_agent.h"
#include "gdsclientsbase.h"
#include "gds_command.h"

GDSClientAuthenticator::GDSClientAuthenticator( GDSClientConnection* connection )
: m_firstPacket( true )
, m_connection( connection )
, m_clientsBase( GDSClientDatabase::instance() )
, m_log( "ClientAuthenticator", LogStream::instance() )
{
	m_connection->connectPacketTarget( this, SLOT( processContent( const QByteArray& ) ) );
	m_connection->connectDestroyedTarget( this, SLOT( connectionDestroyed( QObject * ) ) );
}

void GDSClientAuthenticator::authSuccess( GDSClient* client )
{
	m_log.info( "Authenticate success" );
	authEnded();
//	m_connection->registerUser( client->login() );
	m_connection->connectPacketTarget( client, SLOT( processData( const QByteArray& ) ) );
	m_connection->newPacket( m_packet );
}

void GDSClientAuthenticator::authEnded()
{
	m_connection->disconnectPacketTarget( this, SLOT( processContent( const QByteArray& ) ) );
	m_connection->disconnectDestroyedTarget( this, SLOT( connectionDestroyed( QObject * ) ) );
	deleteLater();
}

void GDSClientAuthenticator::authFailed()
{
	m_log.info( "Authenticate failed" );
	m_connection->sendContent( QString( "** SimpleDS wrong credentials **\n" ).toUpper().toLocal8Bit() );
	authEnded();
	m_connection->closeConnection();
}

void GDSClientAuthenticator::processContent( const QByteArray& packet )
{
	m_log.proto( "Incoming content:", packet );
	if ( m_firstPacket )
	{
		m_firstPacket = false;
		m_packet = packet;
		processMessage( packet );
	}
	else
	{
		m_log.warn( "Connection is not authorized yet. Dropping the packet." );
	}
}

void GDSClientAuthenticator::connectionDestroyed( QObject * )
{
	m_log.info( "Connection destroyed before validation" );
//	m_connection->closeConnection();
	deleteLater();
}

void GDSClientAuthenticator::processMessage( const QByteArray& packet )
{
	GDSClient* client = NULL;
	GDSCommand* c = GDSCommandCreator::create( packet );
	try
	{
		if ( dynamic_cast<GDSCommandBatch *>( c ) != NULL )
		{
			client = new GDSClientAirline( m_connection );
		}
		else
		{
			GDSCommandJump* cmd = dynamic_cast<GDSCommandJump *>( c );
			if ( cmd != NULL )
			{
				GDSClientAuthInfo *auth = GDSClientDatabase::instance()->getAuthByLogin( cmd->login() );
				if ( auth && auth->password() == cmd->password() )
					client = new GDSClientAgent( m_connection, cmd->login() );
			}
		}
	}
	catch (InternalError& e)
	{
	}
	
	delete c;
	if ( client )
	{
		authSuccess( client );
	}
	else
	{
		authFailed();
	}
}
