#include <QFile>
#include "gdsdatabase_flights.h"
#include "exception.h"

GDSFlightDatabase* GDSFlightDatabase::m_instance = NULL;
const QString GDSFlightDatabase::filename = QString("flights.db");

GDSFlightDatabase* GDSFlightDatabase::instance()
{
	if ( !m_instance )
		m_instance = new GDSFlightDatabase;
	return m_instance;
}

GDSFlightDatabase::GDSFlightDatabase() {
    maxrecords = 10000;
    data = QByteArray(recordlen * maxrecords, ' ');
    restore();
}

void GDSFlightDatabase::add(const GDSFlight &flight) {
    QByteArray flight_string = flight.toString().toAscii();

    for(int record=0; record<maxrecords; record++) {
        // find the free space
        if(data[record * recordlen] != ' ')
            continue;
        memcpy(data.data() + record * recordlen, flight_string.data(),
               flight_string.length());
        break;
    }
}

GDSFlight * GDSFlightDatabase::get(long id) {
	check( id <= maxrecords );
    return new GDSFlight(QByteArray(data.data() + id * recordlen, recordlen) );
}

bool GDSFlightDatabase::compareWithTemplate( const GDSFlight& f, const GDSFlight& templ )
{
	bool result = true;
	result &= ( templ.flight_num.isEmpty() || f.flight_num == templ.flight_num );
	result &= ( templ.date.isNull() || f.date.date() == templ.date.date() );
	result &= ( templ.airline.isEmpty() || f.airline == templ.airline );
	result &= ( templ.orig.isEmpty() || f.orig == templ.orig );
	result &= ( templ.dest.isEmpty() || f.dest == templ.dest );
	result &= ( f.seats.exist() );
	return result;
}

QList<long> GDSFlightDatabase::getByTemplate(const GDSFlight& templ) {
    QList<long> ret;
    for (int record=0; record<maxrecords; record++) {
        if(data[record * recordlen] == ' ')
            continue;

        GDSFlight f(QByteArray(data.data() + record * recordlen, recordlen) );
        if(compareWithTemplate(f,templ))
            ret.append(record);
		if (ret.length() >= 50)
			break;
    }
    return ret;
}

QList<long> GDSFlightDatabase::getByODD(const QString& orig, const QString& dest, const BAYDateTime& date) {
	GDSFlight templ;
	templ.setOrig( orig );
	templ.setDest( dest );
	templ.setDate( date );
	return getByTemplate( templ );
}

void GDSFlightDatabase::update(long id, GDSFlight *flight) {
	check( id <= maxrecords );
    QByteArray flight_string = flight->toString().toAscii();
	memcpy( data.data() + id * recordlen, flight_string.data(), flight_string.length() );
}

bool GDSFlightDatabase::save() {
    QFile file_db(filename);

    if (!file_db.open(QIODevice::WriteOnly | QIODevice::Truncate))
         return false;

    file_db.write( data.data(), data.size() );
    return true;
}

bool GDSFlightDatabase::restore() {
    QFile file_db(filename);

    if (!file_db.open(QIODevice::ReadOnly))
         return false;

    file_db.read( data.data(), data.size() );
    return true;
}
