#include <QFile>
#include "gdsdatabase_hotels.h"
#include "exception.h"

GDSHotelDatabase* GDSHotelDatabase::m_instance = NULL;

const QString GDSHotelDatabase::filename = QString("hotels.db");

GDSHotelDatabase* GDSHotelDatabase::instance()
{
	if ( !m_instance )
		m_instance = new GDSHotelDatabase;
	return m_instance;
}

GDSHotelDatabase::GDSHotelDatabase() {
    maxrecords = 10000;
    data = QByteArray(recordlen * maxrecords, ' ');
    restore();
}

void GDSHotelDatabase::add(const GDSHotel &hotels) {
    QByteArray hotelsstring = hotels.toString().toAscii();

    for(int record=0; record<maxrecords; record++) {
        // find the free space
        if(data[record * recordlen] != ' ')
            continue;
        memcpy(data.data() + record * recordlen, hotelsstring.data(),
               hotelsstring.length());
        break;
    }
}

GDSHotel * GDSHotelDatabase::get(long id) {
	check( id <= maxrecords );
    return new GDSHotel(QByteArray(data.data() + id * recordlen, recordlen) );
}

bool GDSHotelDatabase::compareWithTemplate( const GDSHotel& f, const GDSHotel& templ )
{
    bool result = true;
    result &= ( templ.date.isNull() || f.date == templ.date );
    result &= ( templ.chain.isEmpty() || f.chain == templ.chain );
    result &= ( templ.name.isEmpty() || f.name == templ.name );
    result &= ( templ.city.isEmpty() || f.city == templ.city );
    result &= ( f.rooms.exist() );
    return result;
}

QList<long> GDSHotelDatabase::getByTemplate(const GDSHotel& templ) {
    QList<long> ret;
    for (int record=0; record<maxrecords; record++) {
        if(data[record * recordlen] == ' ')
            continue;

        GDSHotel f(QByteArray(data.data() + record * recordlen, recordlen) );
        if(compareWithTemplate(f,templ)) {
            ret.append(record);
        }
		if (ret.length() >= 50)
			break;
    }
    return ret;
}

QList<long> GDSHotelDatabase::getByCD(const QString& city, const BAYDateTime& date) {
	GDSHotel templ;
	templ.setDate( date );
	templ.setCity( city );
    return getByTemplate( templ );
}

void GDSHotelDatabase::update(long id, GDSHotel* hotel) {
	check( id <= maxrecords );
    QByteArray hotel_string = hotel->toString().toAscii();
	memcpy( data.data() + id * recordlen, hotel_string.data(), hotel_string.length() );
}

bool GDSHotelDatabase::save() {
    QFile file_db(filename);

    if (!file_db.open(QIODevice::WriteOnly | QIODevice::Truncate))
         return false;

    file_db.write( data.data(), data.size() );
    return true;
}

bool GDSHotelDatabase::restore() {
    QFile file_db(filename);

    if (!file_db.open(QIODevice::ReadOnly))
         return false;

    file_db.read( data.data(), data.size() );
    return true;
}
