#include <QFile>
#include "gdsdatabase_pnrs.h"
#include "gdspnr.h"
#include "exception.h"

GDSPnrDatabase* GDSPnrDatabase::m_instance = NULL;
const QString GDSPnrDatabase::filename = QString("pnrs.db");

GDSPnrDatabase* GDSPnrDatabase::instance()
{
	if ( !m_instance )
		m_instance = new GDSPnrDatabase;
	return m_instance;
}

GDSPnrDatabase::GDSPnrDatabase() {
	m_lastRecord = 0;
    maxrecords = 1000000;
    m_data = QByteArray(recordlen * maxrecords, ' ');
    restore();
}

void GDSPnrDatabase::add(const GDSPnr &pnrs) {
    QByteArray pnrsstring = pnrs.toString().toAscii();
    for(int record=0; record<maxrecords; record++) {
        // find the free space
        if(m_data[record * recordlen] != ' ')
            continue;
        memcpy(m_data.data() + record * recordlen, pnrsstring.data(),
               pnrsstring.length());
		if ( record > m_lastRecord )
			m_lastRecord = record;
        break;
    }
}

GDSPnr * GDSPnrDatabase::get(long id) {
	check( id <= maxrecords );
    return new GDSPnr(QByteArray(m_data.data() + id * recordlen, recordlen) );
}

bool GDSPnrDatabase::compareWithTemplate( const GDSPnr& f, const GDSPnr& templ )
{
    bool result = true;
    result &= ( templ.name.isEmpty() || f.name == templ.name );
    result &= ( templ.surname.isEmpty() || f.surname == templ.surname );
    result &= ( templ.passport.isEmpty() || f.passport == templ.passport );
	result &= ( templ.agent.isEmpty() || f.agent == templ.agent );
	result &= ( templ.uniqID.isEmpty() || f.uniqID == templ.uniqID );
    return result;
}

QList<int> GDSPnrDatabase::getByTemplate(const GDSPnr& templ) {
    QList<int> ret;
    for (int record=0; record<maxrecords; record++) {
        if(m_data[record * recordlen] == ' ')
            continue;
        GDSPnr f(QByteArray(m_data.data() + record * recordlen, recordlen) );
        if(compareWithTemplate(f,templ)) {
            ret.append(record);
        }
    }
    return ret;
}

int GDSPnrDatabase::deleteByTemplate(const GDSPnr& templ) {
	int ret = 0;
	for (int record=0; record<maxrecords; record++) {
		if(m_data[record * recordlen] == ' ')
			continue;

		GDSPnr f(QByteArray(m_data.data() + record * recordlen, recordlen) );
		if(compareWithTemplate(f,templ)) {
			ret++;
			m_data[record * recordlen] = ' ';
		}
	}
	return ret;
}

QList<int> GDSPnrDatabase::getByAID(const QString& id, const QString& agentId) {
	GDSPnr templ;
	templ.setAgentId( agentId );
	templ.setUniqID( id );
	return getByTemplate( templ );
}

bool GDSPnrDatabase::save() {
    QFile file_db(filename);

    if (!file_db.open(QIODevice::WriteOnly | QIODevice::Truncate))
         return false;

    file_db.write( m_data.data(), (m_lastRecord + 1) * recordlen );
    return true;
}

bool GDSPnrDatabase::restore() {
    QFile file_db(filename);

    if (!file_db.open(QIODevice::ReadOnly))
         return false;

    file_db.read( m_data.data(), m_data.size() );
    return true;
}

int GDSPnrDatabase::deleteByAID( const QString& id, const QString& agentId )
{
	GDSPnr templ;
	templ.setAgentId( agentId );
	templ.setUniqID( id );
	return deleteByTemplate( templ );
}
