#include <QStringList>

#include "gdsflight.h"
#include "gds_command.h"
#include "exception.h"

GDSFlight::GDSFlight()
{
}

GDSFlight::GDSFlight(const QString& serialized) {
    QStringList fields = serialized.split('\t');
	this->airline = fields.value(0).trimmed();
	this->flight_num = fields.value(1).trimmed();
	this->orig = fields.value(2).trimmed();
	this->dest = fields.value(3).trimmed();
	this->date = BAYDateTime(fields.value(4).trimmed());
	this->seats = GDSPositions(fields.value(5).trimmed());
}

GDSFlight::GDSFlight(GDSCommandAddFlight* cmd) {
	this->airline = cmd->airline();
	this->flight_num = cmd->flightNum();
	this->orig = cmd->orig();
	this->dest = cmd->dest();
	this->date = cmd->date();
	this->seats = cmd->seats();
}

GDSFlight::GDSFlight(GDSCommandSellFlight* cmd) {
	this->airline = cmd->carrier();
	this->flight_num = cmd->flight();
	this->orig = cmd->origin();
	this->dest = cmd->destination();
	this->date = cmd->date();
}

QString GDSFlight::toString() const
{
	QStringList ret;
	ret << airline << flight_num << orig << dest << date.toString() << seats.toString();
	QString result = ret.join( "\t" );
	check(result.length() < 512);
	return result;
}

QString GDSFlight::toPrettyString() const
{
	return QString("%1 %2  %3  %4  %5  %6").arg( airline, 2 ).arg( flight_num, 4 ).arg( seats.toStringPretty(), 20 ).arg( orig, 3 ).arg( dest, 3 ).arg( date.toStringTime(), 4 );
}

void GDSFlight::setClass( const QString& sclass )
{
	m_class = sclass;
}

QString GDSFlight::toShortPrettyString() const
{
	return QString("%1 %2 %3 %4 %5%6").arg( airline, 2 ).arg( flight_num, 4 ).arg( m_class, 1 ).arg( date.toString(), 4 ).arg( orig, 3 ).arg( dest, 3 );
}

void GDSFlight::setOrig( const QString& orig )
{
	this->orig = orig;
}

void GDSFlight::setDest( const QString& dest )
{
	this->dest = dest;
}

void GDSFlight::setDate( const BAYDateTime& date )
{
	this->date = date;
}

int GDSFlight::getAvailableSeats()
{
	return seats.getAvail( m_class );
}

void GDSFlight::decAvailableSeats( int n )
{
	seats.decAvail( m_class, n );
}

void GDSFlight::incAvailableSeats( int n )
{
	seats.incAvail( m_class, n );
}
