#include <QStringList>

#include "gdshotel.h"
#include "gds_command.h"
#include "exception.h"

GDSHotel::GDSHotel()
{
}

GDSHotel::GDSHotel(GDSCommandAddHotel* cmd)
{
	date = cmd->date();
	chain = cmd->chain();
	name = cmd->name();
	city = cmd->city();
	rooms = cmd->rooms();
}

GDSHotel::GDSHotel(const QString& serialized) {
    QStringList fields = serialized.split('\t');
	this->date = BAYDateTime( fields.value(0).trimmed() );
	this->chain = fields.value(1).trimmed();
	this->name = fields.value(2).trimmed();
	this->city = fields.value(3).trimmed();
	this->rooms = GDSPositions(fields.value(4).trimmed());
}

QString GDSHotel::toString() const {
	QStringList ret;
	ret << date.toString() << chain << name << city << rooms.toString();
	QString result = ret.join( "\t" );
	check(result.length() < 512);
	return result;
}

QString GDSHotel::toPrettyString() const {
	return QString("%1/%2  %3").arg( chain, 10 ).arg( name, 20 ).arg( rooms.toStringPretty(), 20 );
}

QString GDSHotel::toShortPrettyString() const
{
	return QString("%1 %2 %3 %4 %5").arg( chain, 2 ).arg( name, 4 ).arg( m_class, 1 ).arg( date.toString(), 4 ).arg( city, 3 );
}

void GDSHotel::setClass( const QString& sclass )
{
	m_class = sclass;
}

void GDSHotel::setDate( const BAYDateTime& date )
{
	this->date = date;
}

void GDSHotel::setCity( const QString& city )
{
	this->city = city;
}

int GDSHotel::getAvailableRooms()
{
	return rooms.getAvail( m_class );
}

void GDSHotel::decAvailableRooms( int n )
{
	rooms.decAvail( m_class, n );
}

void GDSHotel::incAvailableRooms( int n )
{
	rooms.incAvail( m_class, n );
}
