#include <QStringList>

#include "gdspnr.h"
#include "gds_command.h"
#include "exception.h"
#include "gdsdatabase_flights.h"
#include "gdsdatabase_hotels.h"

GDSPnr::GDSPnr()
{
}

GDSPnr::GDSPnr(const QString& serialized) {
    QStringList fields = serialized.split('\t');
	this->uniqID = fields.value(0).trimmed();
	this->agent = fields.value(1).trimmed();
	this->flightId = fields.value(2).trimmed().toInt();
	this->isAvia = fields.value(3).trimmed().toInt();
	this->m_class = fields.value(4).trimmed();
	this->name = fields.value(5).trimmed();
	this->surname = fields.value(6).trimmed();
	this->title = fields.value(7).trimmed();
	this->passport = fields.value(8).trimmed();
}

GDSPnr::GDSPnr( const long flight_or_hotel_id, GDSCommandAddPassenger* c, const QString& agent, bool isAvia )
{
	this->name = c->name();
	this->surname = c->lastName();
	this->title = c->title();
	this->passport = c->passport();
	this->agent = agent;
	this->flightId = flight_or_hotel_id;
	this->isAvia = isAvia ? 1 : 0;
}

QString GDSPnr::toString() const {
	QStringList ret;
	check( name.length() <= 55 );
	check( surname.length() <= 55 );
	check( title.length() <= 5 );
	check( passport.length() <= 70 );
	ret << uniqID << agent << QString::number(flightId) << QString::number(isAvia) << m_class << name << surname << title << passport;
	QString result = ret.join( "\t" );
	check( result.length() <= 170 );
	return result;
}

QString GDSPnr::toPrettyString() const
{
	QString result;
	if ( isAvia )
	{
		GDSFlight *f = GDSFlightDatabase::instance()->get( flightId );
		f->setClass( m_class );
		result = QString("%1 %2 %3 %4 %5 FLT %6").arg(name, surname, title, passport, uniqID, f->toShortPrettyString() );
		delete f;
	}
	else
	{
		GDSHotel *f = GDSHotelDatabase::instance()->get( flightId );
		f->setClass( m_class );
		result = QString("%1 %2 %3 %4 %5 HTL %6").arg(name, surname, title, passport, uniqID, f->toShortPrettyString() );
		delete f;
	}
	return result;
}

QString GDSPnr::toShortPrettyString() const
{
	return QString("%1 %2 %3").arg(name, surname, title);
}

void GDSPnr::setUniqID( const QString& uniqID )
{
	this->uniqID = uniqID;
}

void GDSPnr::setAgentId( const QString& agentId )
{
	this->agent = agentId;
}

void GDSPnr::setClass( const QString& sclass )
{
	this->m_class = sclass;
}
