#include "gdspositions.h"


GDSPosition::GDSPosition( const QString& pclass, int quantity, int price )
{
	this->pclass = pclass;
	this->quantity = quantity;
	this->price = price;
}

GDSPositions::GDSPositions( const QString& string )
{
    QStringList temp( string.split(",") );
	foreach( const QString& position, temp )
	{
		QStringList pos_info = position.split(":");
		if( pos_info.length() != 3 )
			continue;
		m_positions.append( GDSPosition( pos_info[0], pos_info[1].toInt(), pos_info[2].toInt() ) );
	}
}

QString GDSPositions::toString() const
{
	QStringList temp;
	foreach( const GDSPosition& position, m_positions )
	{
		temp.append( QString("%1:%2:%3").arg( position.pclass ).arg( position.quantity ).arg( position.price ) );
	}
    return temp.join(",");
}

QString GDSPositions::toStringPretty() const
{
	QStringList result;
	foreach( const GDSPosition& position, m_positions )
	{
		result.push_back( QString( "%1%2" ).arg( position.pclass ).arg( position.quantity ) );
	}
	return result.join(" ");
}

long GDSPositions::getPrice( const QString& position_class )
{
    foreach( const GDSPosition& position, m_positions )
	{
        if( position_class == position.pclass )
		{
            return position.price;
        }
    }
    return -1;
}

long GDSPositions::getAvail( const QString& position_class )
{
	foreach( const GDSPosition& position, m_positions )
	{
		if( position_class == position.pclass )
		{
			return position.quantity;
		}
	}
    return -1;
}

void GDSPositions::setAvail( const QString& position_class, long avail )
{
	for ( int i = 0; i < m_positions.length(); i++ )
	{
		if( position_class == m_positions[i].pclass )
		{
			m_positions[i].quantity = avail;
		}
	}
}

bool GDSPositions::isEmpty() const
{
    return m_positions.isEmpty();
}

void GDSPositions::decAvail( const QString& position_class, long n )
{
	setAvail( position_class, getAvail( position_class ) - n );
}

void GDSPositions::incAvail( const QString& position_class, int n )
{
	setAvail( position_class, getAvail( position_class ) + n );
}

bool GDSPositions::exist() const
{
	foreach( const GDSPosition& pos, m_positions )
	{
		if ( pos.quantity > 0 )
			return true;
	}
	return false;
}
