#include "logger.h"
#include "logstream.h"

LogLevel* Logger::errorLL = addLogLevel( "ERROR" );
LogLevel* Logger::warnLL  = addLogLevel( "WARN " );
LogLevel* Logger::infoLL  = addLogLevel( "INFO " );
LogLevel* Logger::protoLL = addLogLevel( "PROTO" );

LogLevel* Logger::addLogLevel( const QString& name )
{
	return LogStream::addLogLevel( name );
}

Logger::Logger( const QString& description, LogStream* logstream )
: m_description( description )
, m_logstream( logstream )
{
}

void Logger::setDescription( const QString& description  )
{
	m_description = description;
}

Logger::Logger( const Logger& rhs )
: m_description( rhs.m_description )
, m_logstream( rhs.m_logstream )
{
}

Logger::~Logger()
{
}

Logger& Logger::operator=( const Logger& rhs )
{
	m_description = rhs.m_description;
	m_logstream = rhs.m_logstream;
	return *this;
}

void Logger::error( const QString& message )
{
	m_logstream->write( errorLL, m_description, message );
}

void Logger::warn( const QString& message )
{
	m_logstream->write( warnLL, m_description, message );
}

void Logger::info( const QString& message )
{
	m_logstream->write( infoLL, m_description, message );
}

void Logger::proto( const QString& message, const QByteArray& data )
{
	m_logstream->write( protoLL, m_description, message, data );
}
