#include "logstream.h"
#include <QDateTime>
#include <QStringList>
#include <QFile>
#include <QBuffer>

LogStream* LogStream::m_instance = NULL;

LogLevel::LogLevel( const QString& name )
: m_name( name )
, m_enabled( false )
{}

QMap< QString, LogLevel* >& LogStream::logLevels( )
{
	static QMap< QString, LogLevel* > m_logLevels;
	return m_logLevels;
}

LogStream::LogStream( const QString& filename )
: m_stream( NULL )
{
	setFile( filename );
}

void LogStream::setFile( const QString& fileName )
{
	QIODevice* newStream = new QFile( fileName );
	if ( !dynamic_cast<QFile*>( newStream )->open( QIODevice::WriteOnly | QIODevice::Append | QIODevice::Unbuffered ) )
	{
		delete newStream;
		return;
	}

	if ( QBuffer* buffer = dynamic_cast< QBuffer* >( m_stream ) )
	{
		buffer->reset();
		QByteArray data = buffer->readAll();
		newStream->write( data );
	}

	if ( m_stream )
	{
		m_stream->close();
		delete m_stream;
	}
	m_stream = newStream;
	m_fileName = fileName;
}

LogStream::~LogStream()
{
	m_stream->close();
	delete m_stream;
}

void LogStream::write( const LogLevel* level, const QString& logger, const QString& line, const QByteArray& content )
{
	if ( !level->enabled() )
		return;
	write( level, logger, line );
#ifdef _WIN32
	QByteArray contentString = content + "\r\n";
#else
	QByteArray contentString = content + "\r\n";
#endif
	m_stream->write( contentString );
}

void LogStream::write( const LogLevel* level, const QString& logger, const QString& line )
{
	if ( !level->enabled() )
		return;

	QString date( QDateTime::currentDateTime().toString( "yyyy/MM/dd hh:mm:ss.zzz" ) );

	QStringList sl = line.split( "\n" );
	for( int i = 0; i < sl.count(); i++ )
		m_stream->write( QString( "%1 %2 [%3] - %4\n" ).arg( date, level->name(), logger, sl[i].trimmed() ).toLocal8Bit() );
}

LogLevel* LogStream::addLogLevel( const QString& name )
{
	QString unifiedName( name.trimmed().toLower() );
	if ( logLevels().find( unifiedName ) == logLevels().end() )
		return ( logLevels()[ unifiedName ] = new LogLevel( name ) );
	else
		return NULL;
}

LogLevel* LogStream::getLogLevel( const QString& name )
{
	QMap< QString, LogLevel* >::const_iterator it = logLevels().constFind( name.trimmed().toLower() );
	if ( it != logLevels().constEnd() )
		return it.value();
	else
		return NULL;
}

LogStream* LogStream::instance()
{
	if ( !m_instance )
		m_instance = new LogStream( "GDSLog.txt" );
	return m_instance;
}
