#ifndef LOGSTREAM_H
#define LOGSTREAM_H

#include <QMap>
#include <QString>

class LogStream;
class QIODevice;

class LogLevel
{
	LogLevel( const QString& name );
public:
	const QString& name() const { return m_name; }
	bool enabled() const { return m_enabled; }
	void enable() { m_enabled = true; }
	void disable() { m_enabled = false; }
private:
	friend class LogStream;
	QString m_name;
	bool m_enabled;
};

class LogStream
{
public:
	virtual ~LogStream();
	static LogStream* instance();
	void write( const LogLevel*, const QString& logger, const QString& line );
	void write( const LogLevel*, const QString& logger, const QString& line, const QByteArray& data );
	void setFile( const QString& fileName );
	void reopen() { setFile( m_fileName ); }

	static LogLevel* addLogLevel( const QString& name );
	static LogLevel* getLogLevel( const QString& name );
	static QMap< QString, LogLevel* >& logLevels();

private:
	LogStream( const QString& filename );
//	LogStream( const LogStream& );
//	LogStream& operator= ( const LogStream& );

	QString m_fileName;
	QIODevice* m_stream;
	static LogStream* m_instance;
};

#endif
